var html = require('bel')
var inWindow = require('in-window')
var morph = inWindow && require('morphdom')
var extend = require('xtend')
var noop = require('noope')
var onload = require('on-load')
var OL_KEY_ID = onload.KEY_ID
var OL_ATTR_ID = onload.KEY_ATTR
var _count = 0

module.exports = function Component (options) {
  options = options || {}
  var createElement = options.createElement || noop
  var shouldUpdate = options.shouldUpdate || noop
  var childrenOnly = options.childrenOnly || false
  var morphOptions = {
    getNodeKey: getNodeKey,
    childrenOnly: childrenOnly
  }
  var load = options.load
  var unload = options.unload
  var loadID

  return (function instance () {
    var key = _count++
    var state = {}
    var element

    function init (state) {
      element = create(state)
      if (load || unload) {
        if (inWindow) {
          onload(element, load, unload, key)
          loadID = element.dataset[OL_KEY_ID]
        }
      }
      return element
    }

    function create (newState) {
      state = extend(state, newState)
      element = createElement(state, html)
      if (element.dataset) {
        element.dataset.bgn = key
      }
      return element
    }

    function update (newState) {
      element.isSameNode = function (el) {
        return !shouldUpdate(state, newState)
      }
      element = morph(element, create(newState), morphOptions)
      if (loadID) {
        element.setAttribute(OL_ATTR_ID, loadID)
      }
      return element
    }

    function render (newState) {
      if (element && inWindow) {
        if (shouldUpdate(state, newState)) {
          element = update(newState)
        }
      } else {
        element = init(newState)
      }
      return element
    }

    return render
  }())
}

function getNodeKey (node) {
  return node && node.dataset && node.dataset.bgn
}
