var test = require('tape')
var Component = require('./')
var inWindow = require('in-window')

function log (element) {
  if (inWindow) {
    console.log(element.outerHTML)
  } else {
    console.log(element.toString())
  }
}

test('Component', t => {
  t.test('should exist', t => {
    t.ok(Component, 'exists')
    t.end()
  })

  t.test('should return render', t => {
    var render = Component({
      createElement: (state, html) => {
        var title = state.title
        return html`<h1>${title}</h1>`
      },
      shouldUpdate: x => { return false }
    })
    t.ok(render, 'render')
    t.end()
  })

  t.test('should render', t => {
    var render = Component({
      createElement: (state, html) => {
        var title = state.title
        return html`<h1>${title}</h1>`
      },
      shouldUpdate: x => { return false }
    })
    var element = render({title: 'YOLO'})
    t.ok(element, 'rendered', log(element))
    t.end()
  })

  if (inWindow) {
    t.test('should return key', t => {
      var render = Component({
        createElement: (state, html) => {
          var title = state.title
          return html`<h1>${title}</h1>`
        },
        shouldUpdate: x => { return false }
      })
      var element = render({title: 'KEYED'})
      log(element)
      element = render({title: 'KEYED UP'})
      t.ok(element, 'keyed', log(element))
      t.end()
    })
  }

  t.test('should update', t => {
    var render = Component({
      createElement: (state, html) => {
        var title = state.title
        return html`<h1>${title}</h1>`
      },
      shouldUpdate: x => { return false }
    })
    var element = render({title: 'YOLO'})
    element = render({title: 'FOMO'})
    t.ok(element, 'updated', log(element))
    t.end()
  })

  t.test('should not update', t => {
    var render = Component({
      createElement: (state, html) => {
        var title = state.title
        return html`<h1>${title}</h1>`
      },
      shouldUpdate: (n, o) => { return o.title !== n.title }
    })
    var element = render({title: 'YOLO'})
    var newElement = render({title: 'YOLO'})
    t.deepEqual(newElement, element, 'same element', log(element))
    t.end()
  })
})

